// Importing modules from D3
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  axisLeft,
  axisBottom,
  line,
  max
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => { 
  // Assigning X axis data   
  const xValue = d => d.Year; // Accessor function for x value
  const xAxisLabel = 'Years'; // X axis label
  // Assigning Y axis data 
  const yValue = d => d.Units_Sold; // Accessor function for y value
  const yAxisLabel = 'Units Sold (M)'; // Y axis label
  
  const margin = { top: 40, right: 20, bottom: 55, left: 130 }; // Margin object
  const innerWidth = width - margin.left - margin.right; // Width of the chart area
  const innerHeight = height - margin.top - margin.bottom; // Height of the chart area
  
  //  Scale for X axis
  const xScale = scaleTime()
    .domain(extent(data, xValue)) // Set the domain of x scale
    .range([0, innerWidth]) // Set the range of x scale
    .nice(); // Round values to nice numbers
  
  //  Scale for Y axis  
  const yScale = scaleLinear()
    .domain([0, max(data, yValue )]) // Set the domain of y scale
    .range([innerHeight, 0]) // Set the range of y scale
    .nice(); // Round values to nice numbers

  // Placing the svg element  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`); // Translate g to margin
  
  const xAxis = axisBottom(xScale) // Create X axis
    .tickSize(-innerHeight) // Set tick size
    .tickPadding(15); // Set tick padding
  
  const yAxis = axisLeft(yScale) // Create Y axis
    .tickSize(-innerWidth) // Set tick size
    .tickPadding(15); // Set tick padding
  	
  const yAxisG = g.append('g').call(yAxis); // Append Y axis to the chart
  yAxisG.selectAll('.domain').remove(); // Remove horizontal line in the Y axis
  
  yAxisG.append('text') // Append Y axis label
    .attr('class', 'axis-label')
    .attr('y', -90)
    .attr('x', -innerHeight / 6)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis) // Append X axis to the chart
  	.attr('transform', `translate(0,${innerHeight})`); // Translate X axis to bottom
  
  xAxisG.select('.domain').remove(); // Remove vertical line in the X axis
  
  xAxisG.append('text') // Append X axis label
    .attr('class', 'axis-label')
    .attr('y', 80)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

// Drawing a line
//adding it to svg 
  
// Creating a line generator function using the 'line' method from D3
const lineGenerator = line()
  .x(d => xScale(xValue(d))) // Defining the x position of the line by mapping the data to the xScale
  .y(d => yScale(yValue(d))); // Defining the y position of the line by mapping the data to the yScale
  
// Adding a path element to the 'g' element created earlier
g.append('path')
  .attr('class', 'line_path') // Assigning a class for styling in CSS
  .attr('d', lineGenerator(data)); // Using the line generator function to generate the path data for the line

};

// Loading CSV data
csv('https://gist.githubusercontent.com/anvesh2129/b13f9947cdd1e05b694b8ee00c07e2c6/raw/5995b609e1f484ac14a3801fb424c92107a8a8bf/xbox_yearly_sales')
  .then(data => {
    data.forEach(d => {
    	d.Units_Sold = +d.Units_Sold;
      //Formatting to retrieve year from the date
      d.Year = d3.timeParse("%Y")(d.Year);
    });

    render(data);
  });